{ 25/4/97
	TMemoryStream does not allow dynamic expansion of it's buffer
	while keeping it's contents intact, TDynamicStream does. }

unit DynamicStream;

interface

type
	TDynamicStream = class(TObject)
	private
		FStore: String;
		FPosition: Integer;
		FSize: Integer;
		function PositionRead: Integer;
		procedure PositionWrite(value: Integer);
		function GetSize: Integer;
	public
		Memory: PChar;
		property Position: Integer read PositionRead write PositionWrite;
		property Size: Integer read GetSize;
		constructor Create;
		procedure SetSize(Size: Integer);
		procedure Write(const Buf; Count: Integer);
		function Read(var Buf; Count: Integer): Integer;
		procedure Clear;
	end;

implementation

constructor TDynamicStream.Create;
begin
	Clear;
end;

procedure TDynamicStream.SetSize(Size: Integer);
begin
	// expand or shrink the stream
	SetLength(FStore, Size+1);
	FSize := Size;
	Memory := @FStore[1];
	if FPosition > FSize then
		FPosition := FSize;
end;

procedure TDynamicStream.Clear;
begin
	FPosition := 0;
	FSize := 0;
	SetSize(0);
	Memory := nil;
end;

function TDynamicStream.GetSize: Integer;
begin
	result := FSize;
end;

procedure TDynamicStream.Write(const Buf; Count: Integer);
begin
	// automatically expand the stream to new size if required
	if Count+FPosition > FSize then
		SetSize(Count+FPosition);
	Move(Buf, (@FStore[FPosition+1])^, Count);
	inc(FPosition, Count);
end;

function TDynamicStream.Read(var Buf; Count: Integer): Integer;
begin
	if Count+FPosition > FSize then
		Count := FSize-FPosition+1;
	Move((@FStore[FPosition+1])^, Buf, Count);
	inc(FPosition, Count);
	result := Count;
end;


function TDynamicStream.PositionRead: Integer;
begin
	result := FPosition;
end;

procedure TDynamicStream.PositionWrite(value: Integer);
begin
	if value > FSize then
		value := FSize;
	FPosition := value;
end;

end.
