// Miscellaneous routunes for various takes - uses TDynamicStream

unit POSUtilities;

interface

uses
	SysUtils, DB, DBTables, DynamicStream;

	procedure PackRecord(var AStream: TDynamicStream; tblData: TDataSet);
	function UnpackRecord(pData: PChar; tblData: TDataSet): Integer;
	function ConvertCurrency(sText: String): String;
	function PadStrRight(sText: String; nLen: Integer): String;
	function PadMoneyRightLine(sLine: String; nLineLen: Integer;
		cMoney: Currency): String;
	function PadRight(Value: double; nLen: Integer): String;
	function FormatMoney(cPrice: Currency; nLen: Integer; bSymbol: Boolean): String;
	function GetBlobText(var sText: String; sFieldName: String;
		tblRead: TDataSet): String;
	procedure SetBlobText(sText, sFieldName: String; tblWrite: TDataSet);

implementation

uses
	POSText;

procedure PackRecord(var AStream: TDynamicStream; tblData: TDataSet);
var
	a, nSize: Integer;
	tblStream: TBlobStream;
begin
	// Pack all fields for the current record in tblData into AStream
	for a:=0 to tblData.FieldCount-1 do
	with tblData do
	begin
		if Fields[a].DataType <> ftAutoInc then
		begin
			if Fields[a].DataType in [ftMemo, ftBlob, ftGraphic, ftFmtMemo] then
			begin
				tblStream := TBlobStream.Create(Fields[a] as TBlobField, bmRead);
				AStream.SetSize(AStream.Size + SizeOf(Integer) + tblStream.Size);
				// write blob's size
				nSize := tblStream.Size;
				AStream.Write(nSize, SizeOf(Integer));
				// write blob's content
				tblStream.Read((AStream.Memory+AStream.Position)^, tblStream.Size);
				AStream.Position := AStream.Position+tblStream.Size;
					tblStream.Free;
			end
			else
			begin
				AStream.SetSize(AStream.Size+Fields[a].DataSize);
				if not Fields[a].GetData(PChar(AStream.Memory)+AStream.Position) then
					FillChar((PChar(AStream.Memory)+AStream.Position)^,	Fields[a].DataSize, #0);
				AStream.Position := AStream.Position + Fields[a].DataSize;
			end;
		end;
	end;
end;

function UnpackRecord(pData: PChar; tblData: TDataSet): Integer;
var
	nSize, a: Integer;
	tblStream: TBlobStream;
begin
	{ Strip off each field from pData and load it into tblData.
		Record pack layout: FieldSize|FieldData|FieldSize|FieldData|... }
	result := 0;
	for a:=0 to tblData.FieldCount-1 do
	with tblData do
	begin
		if Fields[a].DataType <> ftAutoInc then
		begin
			if Fields[a].DataType in [ftMemo, ftBlob, ftGraphic, ftFmtMemo] then
			begin
				tblStream := TBlobStream.Create(Fields[a] as TBlobField, bmWrite);
				Move((pData+result)^, nSize, SizeOf(Integer));
				inc(result, Sizeof(Integer));
				tblStream.Write((pData+result)^, nSize);
				inc(result, nSize);
				tblStream.Free;
			end
			else
			begin
				Fields[a].SetData(pData+result);
				inc(result, Fields[a].DataSize);
			end;
		end;
	end;
end;

function ConvertCurrency(sText: String): String;
var
	nPos: Integer;
begin
	// Convert all  simbols to the local currency symbol
	if Pos('', sText) > 0 then
		// set the currency string
		for nPos:=1 to Length(sText) do
			if sText[nPos] = '' then
				sText[nPos] := CurrencyString[1];
	result := sText;
end;

function PadMoneyRightLine(sLine: String; nLineLen: Integer;
	cMoney: Currency): String;
var
	sPad, sMoney: String;
begin
	{ Expand line to nLineLen long, padded with spaces, then insert cMoney
		into the right hand margin of the line's string. Account type reports. }
	SetLength(sPad, nLineLen);
	FillChar(PChar(sPad)^, nLineLen, ' ');
	sMoney := FloatToStrF(cMoney, ffCurrency, C_MONEY_WIDTH, 2);
	if sMoney[1] = '-' then
		sMoney := '-'+Copy(sMoney, 3, Length(sMoney)-1)
	else
		sMoney := Copy(sMoney, 2, Length(sMoney)-1);
	result := sLine+Copy(sPad, 1, nLineLen-Length(sLine)-Length(sMoney))+sMoney;
end;

function PadStrRight(sText: String; nLen: Integer): String;
var
	sPad: String;
begin
	SetLength(sPad, nLen);
	FillChar(PChar(sPad)^, nLen, ' ');
	result := Copy(sPad, 1, nLen-Length(sText))+sText;
end;

function PadRight(Value: double; nLen: Integer): String;
var
	sPad: String;
begin
	SetLength(sPad, nLen);
	FillChar(PChar(sPad)^, nLen, ' ');
	result := Copy(sPad, 1, nLen-Length(FloatToStr(Value)))+FloatToStr(Value);
end;

function FormatMoney(cPrice: Currency; nLen: Integer; bSymbol: Boolean): String;
var
	sPad: String;
begin
	{ Right justify money using Windows settings for currency. Pass nLen as
		-1 to return the string with no padding. }
	if nLen > 0 then
	begin
		SetLength(sPad, nLen);
		FillChar(PChar(sPad)^, nLen, ' ');
	end;
	result := FloatToStrF(cPrice, ffCurrency, C_MONEY_WIDTH, 2);
	if not bSymbol then
	begin
		if result[1] = '-' then
			result := '-'+Copy(result, 3, Length(result)-1)
		else
			result := Copy(result, 2, Length(result)-1);
	end;
	if nLen > 0 then
		result := Copy(sPad, 1, nLen-Length(result))+result;
end;

function GetBlobText(var sText: String; sFieldName: String;
	tblRead: TDataSet): String;
var
	tblStream: TBlobStream;
begin
	tblStream := TBlobStream.Create(tblRead.FieldByName(sFieldName)
		as TBlobField, bmRead);
	SetLength(sText, tblStream.Size);
	tblStream.Read(PChar(sText)^, tblStream.Size);
	tblStream.Free;
	result := sText;
end;

procedure SetBlobText(sText, sFieldName: String; tblWrite: TDataSet);
var
	tblStream: TBlobStream;
	bEdit: Boolean;
begin
	if not (tblWrite.State in [dsEdit, dsInsert]) then
	begin
		tblWrite.Edit;
		bEdit := True;
	end
	else
		bEdit := False;
	tblStream :=TBlobStream.Create(tblWrite.FieldByName(sFieldName)
		as TBlobField, bmWrite);
	tblStream.Write(PChar(sText)^, Length(sText)+1);
	tblStream.Free;
	tblWrite.Post;
	if bEdit then
		tblWrite.Edit;
end;

end.
